import java.io.*;
import java.util.regex.*;
public class ExportDBtoOWL {

	private Database db;
	private String saveFile = "shema.owl";
	Config owlTemplate;

	/** ***************************************************************************************************** */
	public ExportDBtoOWL(Database db, String saveFile) throws Exception {
		this.db = db;
		this.saveFile = saveFile;
		db.getConnection();
		ExportSchema();
	}
	/** ***************************************************************************************************** */
	public void ExportSchema() throws Exception {
		long timeStart = System.currentTimeMillis();		
		System.out.println("[+] Exporting Schema ");
		owlTemplate = new Config("SchemaOWL.properties");
		FileWriter schemaFile = new FileWriter(this.saveFile);
		schemaFile.write(Header());
		schemaFile.write(DBdesc());
		String tnames[] = db.getTableNames();
		for (int i = 0; i < tnames.length; i++)
			schemaFile.write(TableDesc(tnames[i]));
		for (int i = 0; i < tnames.length; i++)
		{	
			//System.err.println(i+":"+tnames[i] );
			schemaFile.write(ColumnDesc(tnames[i]));
		}
		schemaFile.write(Footer());
		schemaFile.close();
		float duration = (System.currentTimeMillis()-timeStart)/1000;
		System.out.println("[+] Exporting Schema :" +duration+"s");
	}
	/**
	 * @return the generic header of the OWL Document
	 */
	public String Header() {
		return (owlTemplate.getProperty("Header"));
	}
	/**
	 * @return the generic footer of the OWL Document
	 */
	public String Footer() {
		return (owlTemplate.getProperty("Footer"));
	}
	/** ***************************************************************************************************** */
	public String DBdesc() {
		String outputStr = "";
		outputStr += Replace(owlTemplate.getProperty("DatabaseH"), "DATABASE_NAME", db.getSchemaName());
		String tnames[] = db.getTableNames();
		for (int i = 0; i < tnames.length; i++)
			outputStr += Replace(owlTemplate.getProperty("DatabaseR"), "TABLE_NAME", tnames[i]);
		outputStr += owlTemplate.getProperty("DatabaseF");
		return (outputStr);
	}
	/** ***************************************************************************************************** */
	public String TableDesc(String tableName) throws Exception {
		String outputStr = "";
		outputStr += Replace(owlTemplate.getProperty("TableH"), "TABLE_NAME", tableName);
		String cnames[] = db.getColumnNames(tableName);
		for (int i = 0; i < cnames.length; i++)
			outputStr += Replace(owlTemplate.getProperty("TableR"), "COLUMN_NAME", tableName + "." + cnames[i]);
		// retrieve primary keys
		String pknames[] = db.getPrimaryKeys(tableName);
		if (pknames.length > 0) {
			outputStr += owlTemplate.getProperty("TablePKH");
			for (int i = 0; i < pknames.length; i++)
				outputStr += Replace(owlTemplate.getProperty("TablePKR"), "PK_COLUMN_NAME", tableName + "." + pknames[i]);
			outputStr += owlTemplate.getProperty("TablePKF");
		}
		outputStr += owlTemplate.getProperty("TableF");
		return (outputStr);
	}

	/**
	 * @param tableName
	 * @return the column description of a table (name,foreign key,...)
	 * @throws Exception
	 */
	public String ColumnDesc(String tableName) throws Exception {		
		String outputStr = "";
		String fknames[][] = db.getForeignKeys(tableName);
		String cnames[] = db.getColumnNames(tableName);
		for (int i = 0; i < cnames.length; i++) {
			outputStr += Replace(owlTemplate.getProperty("PropertyH"), "COLUMN_NAME", tableName + "." + cnames[i]);
			outputStr += Replace(owlTemplate.getProperty("PropertyR"), "TABLE_NAME", tableName);
			//TODO : LAST : faire test ds le cas d un length null ?
			for (int j = 0; j < fknames.length; j++) {
				if (cnames[i].equals(fknames[j][0])) {
					//we have a fk
					outputStr += Replace(owlTemplate.getProperty("PropertyFK"), "FK_NAME", fknames[j][1] + "." + fknames[j][2]);
				}
			}

			outputStr += Replace(owlTemplate.getProperty("PropertyT"), "TYPE",db.getColumnType(tableName, cnames[i]));															
			
			//TODO : LENGTH & SCALE PROBLEM FIXED !
			//2004-04-22 : manage the length scale problem									
			if (db.getColumnLength(tableName, cnames[i]) != null )
			{	
				outputStr += Replace(owlTemplate.getProperty("PropertyLength"), "LENGTH",db.getColumnLength(tableName, cnames[i]));
			}
			if (db.getColumnScale(tableName, cnames[i]) != null )
			{	
				outputStr += Replace(owlTemplate.getProperty("PropertyScale"), "SCALE",db.getColumnScale(tableName, cnames[i]));
			}
			outputStr += owlTemplate.getProperty("PropertyF");
		}
		return (outputStr);
	}
	/** ***************************************************************************************************** */
	private String Replace(String inputStr, String inputPattern, String replaceStr) {
		String outputStr = "";
		try {
			Pattern pattern = Pattern.compile(inputPattern);
			Matcher matcher = pattern.matcher(inputStr);
			outputStr = matcher.replaceAll(replaceStr);
		} catch (Exception e) {
			System.out.println("Error Replace ");
		}
		return outputStr;
	}
}
